#ifndef FR_ICM_INC
#define FR_ICM_INC

#include <stdio.h>
#include <math.h>
#include "../ansi_lib_loc/allocate.h"
#include "../ansi_lib_loc/assort.h"
#include "../ansi_lib_loc/multigrid.h"
#include "../ansi_lib_loc/solve2.h"
#include "../ansi_lib_loc/randlib.h"
#include "../ansi_lib_loc/A_comp_dbl_storeflt.h"


#define TOL 1.0e-9
#define WN 0.14644660940673
#define WD 0.10355339059327
#define SG(A,B) ((A) > (B) ? 1.0 : -1.0)



struct cmdline_params{
  char *input;
  char *initial;
  char mask_fname[512];
  char attenuation[512];
  double p;
  int no_iter_param;
  int no_iter_rec;
  char *output;
  int VERB;
  double sigma;
  int interpolate;
  int resol;
  double fov;
  int mask_FLAG;
  int Transcor_FLAG;
}; 



struct all_data{  /* all data except transmission attenuation sino */
  double **X; /* recon image, initially CBP */
  double **Y; /* sinog. counts */
  int **mask;
  double scatter;
  struct sino_info sino;
  int Nsup; /* nb pixels w/ mask not zero */
  int **recon_mask;
};

struct neighbors{
  double NX[4]; /* hor and vert neighbors */
  double DX[4]; /* diagonal neighbors */

};

struct ICMdata{  /* parameters computed during ICM */
  double D1,D2; /* derivatives */
  struct neighbors neigh; 
  double CX; /* center pixel */
  double ML; 
  double SX; /* old est. sigma^p */
  double P;
};


void  f_pro(double *S,struct all_data *data,struct geom_info *geom,double **profiles,struct column *sp_col);
void get_neighborhood(struct ICMdata *ICMdata,struct all_data *data,struct geom_info *geom,int i,int j);
void minmax(double *low, double *high,double ML, double *NX,double *DX);
double diff_cost(double x,void *ICMdata);
double cost(double x,struct ICMdata *ICMdata);
double updateX(struct ICMdata *ICMdata);
double ICMstep(struct ICMdata *ICMdata,struct all_data *data,double *S,int i,int j,struct geom_info *geom,struct column *sp_col,int *FLAG);
void imgcoord(int *out_i,int *out_j,int s,int iter,int N,int M);
int compute_derivatives(struct ICMdata *ICMdata, struct all_data *data, struct column *sp_col,double *S);
void updateS(double *S, struct column *sp_col, double diff);



#endif
